/* LICENSE BLOCK
Part of mde: a Modular D game-oriented Engine
Copyright © 2007-2008 Diggory Hardy

This program is free software: you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation, either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. */

/** Content widgets based on a text block. */
module textContent;

import mde.gui.widget.Widget;
import mde.gui.widget.TextWidget;
import mde.gui.exception;
import mde.gui.renderer.IRenderer;

import mde.content.AStringContent;

debug {
    import tango.util.log.Log : Log, Logger;
    private Logger logger;
    static this () {
	logger = Log.getLogger ("mde.gui.widget.textContent");
    }
}

/// Just displays the content
class DisplayContentWidget : ATextWidget
{
    this (IWidgetManager mgr, widgetID id, WidgetData data, IContent c) {
	WDMinCheck(data, 1);
	content = c;
	if (!content) throw new ContentException ();
	adapter = mgr.renderer.getAdapter (content.toString(0));
	super (mgr, id, data);
    }
    
protected:
    IContent content;
}

/// Capable of editing any ValueContent class
class AStringContentWidget : ATextWidget
{
    this (IWidgetManager mgr, widgetID id, WidgetData data, IContent c) {
        WDMinCheck(data, 1);
        content = cast(AStringContent) c;
        if (!content) //content = new TextContent (null, null);
	    throw new ContentException ();
        adapter = mgr.renderer.getAdapter (content.toString(0));
        super (mgr, id, data);
    }
    
    /** On click, request keyboard input. */
    int clickEvent (wdabs, wdabs, ubyte, bool state) {
	adapter.index = content.editIndex;
	mgr.requestRedraw;
	return 1;	// get keyboard input via keyEvent
    }
    
    void keyEvent (ushort s, char[] i) {
	adapter.text = content.keyStroke (s, i);
	adapter.index = content.editIndex;
	adapter.getDimensions (mw, mh);	// NOTE: only passively change size: next resize will see new minimal size
	mgr.requestRedraw;
    }
    void keyFocusLost () {
	adapter.index;
	content.endEdit;	// update other users of content relying on callbacks
	mgr.requestRedraw;
    }
    
protected:
    AStringContent content;
}
